#include <stdio.h>
#define HOOKDLL_EXPORTS
#include "fxcHook.h"
#include "fxcMessage.h"

static HHOOK g_winhook  = NULL; // hook for window activation
static HHOOK g_apphook  = NULL; // hook for application launch
static HANDLE m_hmodule = NULL; // module handle of the DLL

#define CUBASE_ALWAYS_ON_TOP_WINDOW_BITS    0XFFF00000
#define CUBASE_ALWAYS_ON_TOP_WINDOW_STYLE   0X96C00000
#define CUBASE_ALWAYS_ON_TOP_WINDOW_STYLE_1 0X96CA0000
#define CUBASE_ALWAYS_ON_TOP_WINDOW_STYLE_2 0X96CE0000 
#define CUBASE_ALWAYS_ON_TOP_WINDOW_STYLE_3 0X96CC0000 
#define CUBASE_MESSAGE_BOX_WINDOW_STYLE     0X96C80000


// Hook that looks for "Always On Top" Cubase windows 
// and forces them to ACTUALLY be on top.
LRESULT CALLBACK cubaseWindowHookProc(int code, WPARAM wparam, LPARAM lparam)
{
   if (code == HCBT_ACTIVATE)
   {
      HWND hwnd = (HWND) wparam;

      long style = GetWindowLong(hwnd, GWL_STYLE);
      if ((style & CUBASE_ALWAYS_ON_TOP_WINDOW_BITS) == CUBASE_ALWAYS_ON_TOP_WINDOW_STYLE)
      {
         char classname[1024];
         char title[1024];
         if (GetClassName(hwnd, classname, sizeof(classname)-1) 
               && strcmp(classname, CUBASE_CHILD_WINDOW) == 0
               && GetWindowText(hwnd, title, sizeof(title)-1) 
               && strncmp(title, "MixConsole", 10) != 0)
         {
            //log("target: %s(%x), style: %x\n", title, hwnd, style);
            SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE);
         }
      }
   }

   return CallNextHookEx(g_winhook, code, wparam, lparam);    
}

// Hook that looks for the main Cubase window being created
// so we can set the window hook (above)
LRESULT CALLBACK appLaunchHookProc(int code, WPARAM wparam, LPARAM lparam)
{
   if (code == HSHELL_WINDOWCREATED)
   {
      HWND hwnd = (HWND) wparam;
      char classname[1024];
      if (GetClassName(hwnd, classname, sizeof(classname)-1) && strcmp(classname, CUBASE_MAIN_WINDOW) == 0)
      {
         setCubaseWindowHook(hwnd);
      }
   }

   return CallNextHookEx(g_apphook, code, wparam, lparam);    
}

static void removeCubaseWindowHook()
{
   if (g_winhook != NULL)
   {
      if (!UnhookWindowsHookEx(g_winhook))
      {
         errorMessage("Unable to remove Cubase window hook.\n");
      }
      g_winhook = NULL;
   }
}

static void removeCubaseLaunchWatcherHook()
{
   if (g_apphook != NULL)
   {
      if (!UnhookWindowsHookEx(g_apphook))
      {
         errorMessage("Unable to remove application creation hook.\n");
      }
      g_apphook = NULL;
   }
}

DLLAPI void removeHooks()
{
   removeCubaseWindowHook();
   removeCubaseLaunchWatcherHook();
}


DLLAPI bool setCubaseWindowHook(HWND hwnd)
{
   removeCubaseWindowHook();

   DWORD thread = GetWindowThreadProcessId(hwnd, NULL);
   g_winhook = SetWindowsHookEx(WH_CBT, cubaseWindowHookProc, (HINSTANCE)m_hmodule, thread);
   if (g_winhook == NULL)  
   {
      errorMessage("Unable to set Cubase window hook.\n");
   }
   return g_winhook != NULL;
}

DLLAPI bool setCubaseLaunchWatcherHook()
{
   g_apphook = SetWindowsHookEx(WH_SHELL, appLaunchHookProc, (HINSTANCE)m_hmodule, NULL);
   if (g_apphook == NULL) 
   {
      errorMessage("Unable to set application creation hook.\n");
   }
   return g_apphook != NULL;
}


BOOL APIENTRY DllMain(HANDLE hmodule, DWORD reason, LPVOID reserved)
{
   m_hmodule = hmodule;

   switch (reason) {
      case DLL_PROCESS_ATTACH: 
         break;
      case DLL_PROCESS_DETACH:
         removeCubaseWindowHook();
         removeCubaseLaunchWatcherHook();
         break;
   }
   return TRUE;
}
